Class: StructuredSearch::Tree::From

Inherits:
BaseNode
  • Object
show all
Defined in:
lib/structured_search/tree/from.rb

Overview

FROM reserved word node

Instance Attribute Summary collapse

Attributes inherited from BaseNode

#column, #line, #type

Instance Method Summary collapse

Constructor Details

#initialize(*argv) ⇒ From

:nodoc:



15
16
17
18
# File 'lib/structured_search/tree/from.rb', line 15

def initialize(*argv)
  @sources = {}
  super *argv
end

Instance Attribute Details

#sourcesObject

in sql spec, this is where we store all the derived columns, such as Google, Omniref etc

This is a hash containing the symbol reference and a class reference to what generates the query



12
13
14
# File 'lib/structured_search/tree/from.rb', line 12

def sources
  @sources
end