Class: StructuredSearch::Tree::From
- Defined in:
- lib/structured_search/tree/from.rb
Overview
FROM reserved word node
Instance Attribute Summary collapse
-
#sources ⇒ Object
in sql spec, this is where we store all the derived columns, such as Google, Omniref etc.
Attributes inherited from BaseNode
Instance Method Summary collapse
-
#initialize(*argv) ⇒ From
constructor
:nodoc:.
Constructor Details
#initialize(*argv) ⇒ From
:nodoc:
15 16 17 18 |
# File 'lib/structured_search/tree/from.rb', line 15 def initialize(*argv) @sources = {} super *argv end |
Instance Attribute Details
#sources ⇒ Object
in sql spec, this is where we store all the derived columns, such as Google, Omniref etc
This is a hash containing the symbol reference and a class reference to what generates the query
12 13 14 |
# File 'lib/structured_search/tree/from.rb', line 12 def sources @sources end |